#!/usr/bin/perl
#
# Sensor for gathering iptables performance data for Zabbix
# Copyright (c) 2008 Hrvoje Shute
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Changes and Modifications
# * Thu May  7 21:11:41 CEST 2009
#	- rewritten in perl
#       - rewritten sender to use only one connection

use strict;
use warnings;

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileIPtables";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = ''; 
my $hostname = `/bin/hostname -f`; 
chomp($hostname);

# signal handling
$SIG{'ALRM'} = sub {
        local $SIG{TERM} = 'IGNORE';
        kill TERM => -$$;
        print "0\n";
};

local $SIG{TERM} = sub {
        local $SIG{TERM} = 'IGNORE';
        kill TERM => -$$;
        print "0\n";
};

alarm 25;

# iptables data
my $iptstate="/usr/sbin/iptstate";
my $switch;

# centos or debian
if (-e "/etc/redhat-release") {
  $switch = " -s -t -b s";
} elsif (-e "/etc/debian_version" ){
  $switch = " --single --totals --sort s";
}

my $cmd = "sudo $iptstate $switch";
my $output = qx($cmd);

# get data
my %data;		#	print $output;#  \s+Other:\s+(\d+)
my @tmpArr;
$data{'iptstate.tcp'} = $1 and
$data{'iptstate.udp'} = $2 and
$data{'iptstate.icmp'} = $3 if ($output =~ /Total States[\d|:|\-|\s]+TCP:\s+(\d+)\s+UDP:\s+(\d+)\s+ICMP:\s+(\d+)/i);
$data{'iptstate.other'} = $1 if ($output =~ /\s+Other:\s+(\d+)/i);
@tmpArr = split(/SYN/, $output);
$data{'iptstate.tcp.syn'} = @tmpArr - 1;
@tmpArr = split(/TIME_WAIT/, $output);
$data{'iptstate.tcp.timewait'} = @tmpArr - 1;
@tmpArr = split(/ESTABLISHED/, $output);
$data{'iptstate.tcp.established'} = @tmpArr - 1;
@tmpArr = split(/CLOSE/, $output);
$data{'iptstate.tcp.close'} = @tmpArr - 1;

#print "--$data{'iptstate.other'}!!\n\n";
# write data to variable
foreach my $key (keys %data){
  $inputString .= $hostname ." ". $key ." ". $data{$key} ."\n" if($data{$key}); #$server ." ". $hostname ." ". $port
}

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
# EOF
